package de.telefonbuch.model;

import org.jetbrains.annotations.NotNull;

import java.sql.*;

public class Datenbank {

	private Connection con;

	public Datenbank() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 3);

		try {
			this.connect();
			this.createTables();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	private void createTables() throws SQLException {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 1);

		Statement stmt = this.con.createStatement();
		String createTable, createTrigger, createView;

		createTable = "CREATE TABLE IF NOT EXISTS \"table_vorname\" (" +
				"id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
				"vorname VARCHAR(50) NOT NULL " +
				");";
		stmt.execute(createTable);
		createTable = "CREATE TABLE IF NOT EXISTS \"table_nachname\" (" +
				"id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
				"nachname VARCHAR(50) NOT NULL " +
				");";
		stmt.execute(createTable);
		createTable = "CREATE TABLE IF NOT EXISTS \"table_strasse\" (" +
				"id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
				"strasse VARCHAR(50) NOT NULL " +
				");";
		stmt.execute(createTable);
		createTable = "CREATE TABLE IF NOT EXISTS \"table_hnr\" (" +
				"id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
				"hnr VARCHAR(10) NOT NULL " +
				");";
		stmt.execute(createTable);
		createTable = "CREATE TABLE IF NOT EXISTS \"table_plz\" (" +
				"id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
				"plz VARCHAR(5) NOT NULL " +
				");";
		stmt.execute(createTable);
		createTable = "CREATE TABLE IF NOT EXISTS \"table_ort\" (" +
				"id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL ," +
				"ort VARCHAR(50) NOT NULL " +
				");";
		stmt.execute(createTable);
		createTable = "CREATE TABLE IF NOT EXISTS \"table_telefon\" (" +
				"id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
				"telefon VARCHAR(30) NOT NULL " +
				");";
		stmt.execute(createTable);

		createTable = "CREATE TABLE IF NOT EXISTS \"table_kontakte\" (\n" +
				"    vorname VARCHAR,\n" +
				"    nachname VARCHAR,\n" +
				"    strasse VARCHAR,\n" +
				"    hnr VARCHAR,\n" +
				"    plz VARCHAR,\n" +
				"    ort VARCHAR,\n" +
				"    telefon VARCHAR,\n" +
				"    FOREIGN KEY(vorname) REFERENCES table_vorname(vorname),\n" +
				"    FOREIGN KEY(nachname) REFERENCES table_nachname(nachname),\n" +
				"    FOREIGN KEY(strasse) REFERENCES table_strasse(strasse),\n" +
				"    FOREIGN KEY(hnr) REFERENCES table_hnr(hnr),\n" +
				"    FOREIGN KEY(plz) REFERENCES table_plz(plz),\n" +
				"    FOREIGN KEY(ort) REFERENCES table_ort(ort),\n" +
				"    FOREIGN KEY(telefon) REFERENCES table_telefon(telefon)\n" +
				");";
		stmt.execute(createTable);

		createView = "CREATE VIEW IF NOT EXISTS \"view_kontakte\" AS " +
				"           SELECT" +
				"               table_vorname.vorname, " +
				"               table_nachname.nachname, " +
				"               table_strasse.strasse, " +
				"               table_hnr.hnr, " +
				"               table_plz.plz, " +
				"               table_ort.ort, " +
				"               table_telefon.telefon " +
				"           FROM table_kontakte " +
				"           INNER JOIN table_vorname ON table_kontakte.vorname = table_vorname.id" +
				"           INNER JOIN table_nachname ON table_kontakte.nachname = table_nachname.id" +
				"           INNER JOIN table_strasse ON table_kontakte.strasse = table_strasse.id" +
				"           INNER JOIN table_hnr ON table_kontakte.hnr = table_hnr.id" +
				"           INNER JOIN table_plz ON table_kontakte.plz = table_plz.id" +
				"           INNER JOIN table_ort ON table_kontakte.ort = table_ort.id" +
				"           INNER JOIN table_telefon ON table_kontakte.telefon = table_telefon.id;";
		stmt.execute(createView);

		createTrigger = "CREATE TRIGGER IF NOT EXISTS view_kontakte_INS\n" +
				"INSTEAD OF INSERT ON view_kontakte FOR EACH ROW\n" +
				"BEGIN\n" +
				"   INSERT INTO table_vorname (vorname) SELECT NEW.vorname WHERE NOT EXISTS (SELECT 1 FROM table_vorname WHERE vorname = NEW.vorname);\n" +
				"   INSERT INTO table_nachname (nachname) SELECT NEW.nachname WHERE NOT EXISTS (SELECT 1 FROM table_nachname WHERE nachname = NEW.nachname);\n" +
				"   INSERT INTO table_strasse (strasse) SELECT NEW.strasse WHERE NOT EXISTS (SELECT 1 FROM table_strasse WHERE strasse = NEW.strasse);\n" +
				"   INSERT INTO table_hnr (hnr) SELECT NEW.hnr WHERE NOT EXISTS (SELECT 1 FROM table_hnr WHERE hnr = NEW.hnr);\n" +
				"   INSERT INTO table_plz (plz) SELECT NEW.plz WHERE NOT EXISTS (SELECT 1 FROM table_plz WHERE plz = NEW.plz);\n" +
				"   INSERT INTO table_ort (ort) SELECT NEW.ort WHERE NOT EXISTS (SELECT 1 FROM table_ort WHERE ort = NEW.ort);\n" +
				"   INSERT INTO table_telefon (telefon) SELECT NEW.telefon WHERE NOT EXISTS (SELECT 1 FROM table_telefon WHERE telefon = NEW.telefon);\n" +
				"   INSERT INTO table_kontakte (vorname, nachname, strasse, hnr, plz, ort, telefon)\n" +
				"       SELECT\n" +
				"           (SELECT id FROM table_vorname WHERE vorname = NEW.vorname),\n" +
				"           (SELECT id FROM table_nachname WHERE nachname = NEW.nachname),\n" +
				"           (SELECT id FROM table_strasse WHERE strasse = NEW.strasse),\n" +
				"           (SELECT id FROM table_hnr WHERE hnr = NEW.hnr),\n" +
				"           (SELECT id FROM table_plz WHERE plz = NEW.plz),\n" +
				"           (SELECT id FROM table_ort WHERE ort = NEW.ort),\n" +
				"           (SELECT id FROM table_telefon WHERE telefon = NEW.telefon);\n" +
				"END;";
		stmt.execute(createTrigger);

		createTrigger = "CREATE TRIGGER IF NOT EXISTS view_kontakte_DEL " +
				"   INSTEAD OF DELETE ON view_kontakte FOR EACH ROW " +
				"   BEGIN " +
				"       DELETE FROM table_kontakte" +
				"       WHERE vorname.id = (SELECT id FROM table_vorname WHERE vorname = OLD.vorname)" +
				"           AND nachname.id = (SELECT id FROM table_nachname WHERE nachname = OLD.nachname)" +
				"           AND strasse.id = (SELECT id FROM table_strasse WHERE strasse = OLD.strasse)" +
				"           AND hnr.id = (SELECT id FROM table_hnr WHERE hnr = OLD.hnr)" +
				"           AND plz.id = (SELECT id FROM table_plz WHERE plz = OLD.plz)" +
				"           AND ort.id = (SELECT id FROM table_ort WHERE ort = OLD.ort)" +
				"           AND telefon.id = (SELECT id FROM table_telefon WHERE telefon = OLD.telefon); " +
				"       DELETE FROM table_vorname WHERE id NOT IN (SELECT vorname.id FROM table_kontakte);" +
				"       DELETE FROM table_nachname WHERE id NOT IN (SELECT nachname.id FROM table_kontakte);" +
				"       DELETE FROM table_strasse WHERE id NOT IN (SELECT strasse.id FROM table_kontakte);" +
				"       DELETE FROM table_hnr WHERE id NOT IN (SELECT hnr.id FROM table_kontakte);" +
				"       DELETE FROM table_plz WHERE id NOT IN (SELECT plz.id FROM table_kontakte);" +
				"       DELETE FROM table_ort WHERE id NOT IN (SELECT ort.id FROM table_kontakte);" +
				"       DELETE FROM table_telefon WHERE id NOT IN (SELECT telefon.id FROM table_kontakte);" +
				"   END;";
		stmt.execute(createTrigger);

		createTrigger = "CREATE TRIGGER IF NOT EXISTS view_kontakte_UPD " +
				"   INSTEAD OF UPDATE ON view_kontakte FOR EACH ROW " +
				"   BEGIN " +
				"       INSERT INTO table_vorname (vorname) SELECT NEW.vorname WHERE NOT EXISTS (SELECT 1 FROM table_vorname WHERE vorname = NEW.vorname);" +
				"       INSERT INTO table_nachname (nachname) SELECT NEW.nachname WHERE NOT EXISTS (SELECT 1 FROM table_nachname WHERE nachname = NEW.nachname);" +
				"       INSERT INTO table_strasse (strasse) SELECT NEW.strasse WHERE NOT EXISTS (SELECT 1 FROM table_strasse WHERE strasse = NEW.strasse);" +
				"       INSERT INTO table_hnr (hnr) SELECT NEW.hnr WHERE NOT EXISTS (SELECT 1 FROM table_hnr WHERE hnr = NEW.hnr);" +
				"       INSERT INTO table_plz (plz) SELECT NEW.plz WHERE NOT EXISTS (SELECT 1 FROM table_plz WHERE plz = NEW.plz);" +
				"       INSERT INTO table_ort (ort) SELECT NEW.ort WHERE NOT EXISTS (SELECT 1 FROM table_ort WHERE ort = NEW.ort);" +
				"       INSERT INTO table_telefon (telefon) SELECT NEW.telefon WHERE NOT EXISTS (SELECT 1 FROM table_telefon WHERE telefon = NEW.telefon);" +
				"       UPDATE table_kontakte SET" +
				"           vorname = (SELECT id FROM table_vorname WHERE vorname = NEW.vorname), " +
				"           nachname = (SELECT id FROM table_nachname WHERE nachname = NEW.nachname), " +
				"           strasse = (SELECT id FROM table_strasse WHERE strasse = NEW.strasse), " +
				"           hnr = (SELECT id FROM table_hnr WHERE hnr = NEW.hnr), " +
				"           plz = (SELECT id FROM table_plz WHERE plz = NEW.plz), " +
				"           ort = (SELECT id FROM table_ort WHERE ort = NEW.ort), " +
				"           telefon = (SELECT id FROM table_telefon WHERE telefon = NEW.telefon) " +
				"       WHERE vorname = (SELECT id FROM table_vorname WHERE vorname = OLD.vorname) " +
				"           AND nachname = (SELECT id FROM table_nachname WHERE nachname = OLD.nachname)" +
				"           AND strasse = (SELECT id FROM table_strasse WHERE strasse = OLD.strasse)" +
				"           AND hnr = (SELECT id FROM table_hnr WHERE hnr = OLD.hnr)" +
				"           AND plz = (SELECT id FROM table_plz WHERE plz = OLD.plz)" +
				"           AND ort = (SELECT id FROM table_ort WHERE ort = OLD.ort)" +
				"           AND telefon = (SELECT id FROM table_telefon WHERE telefon = OLD.telefon);" +
				"       DELETE FROM table_vorname WHERE vorname NOT IN (SELECT id FROM table_kontakte);" +
				"       DELETE FROM table_nachname WHERE nachname NOT IN (SELECT id FROM table_kontakte);" +
				"       DELETE FROM table_strasse WHERE strasse NOT IN (SELECT id FROM table_kontakte);" +
				"       DELETE FROM table_hnr WHERE hnr NOT IN (SELECT id FROM table_kontakte);" +
				"       DELETE FROM table_plz WHERE plz NOT IN (SELECT id FROM table_kontakte);" +
				"       DELETE FROM table_ort WHERE ort NOT IN (SELECT id FROM table_kontakte);" +
				"       DELETE FROM table_telefon WHERE telefon NOT IN (SELECT id FROM table_kontakte);" +
				"END;";
		stmt.execute(createTrigger);
	}

	/**
	 * Verbindung zur Datenbank aufbauen
	 */
	private void connect() throws SQLException {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 1);

		String datenbankdatei = "c:\\temp\\telefonbuch.sqlite";

		this.con = DriverManager.getConnection("jdbc:sqlite:" + datenbankdatei);
	}

	public void addKontakt(@NotNull Kontakt kontakt) {

		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 1);

		try {
			Statement stmt = this.con.createStatement();

			String checkKontakt = "SELECT telefon FROM view_kontakte WHERE telefon=\"" + kontakt.getId() + "\";";
			ResultSet checkResult = stmt.executeQuery(checkKontakt);

			int rows = 0;
			while (checkResult.next()) {
				rows++;
			}

			String insertKontakt;

			if (rows == 1) {
				new Fehler().debug("Updating contact", false);

				insertKontakt = "UPDATE view_kontakte SET " +
						"vorname=\"" + kontakt.getVorname() + "\", " +
						"nachname=\"" + kontakt.getNachname() + "\", " +
						"strasse=\"" + kontakt.getStrasse() + "\", " +
						"hnr=\"" + kontakt.getHausnummer() + "\", " +
						"plz=\"" + kontakt.getPlz() + "\", " +
						"ort=\"" + kontakt.getOrt() + "\", " +
						"telefon=\"" + kontakt.getTelefon() + "\" " +
						" WHERE telefon=\"" + kontakt.getId() + "\";";
			} else {
				new Fehler().debug("Insert new contact", false);

				insertKontakt = "INSERT INTO view_kontakte (vorname, nachname, strasse, hnr, plz, ort, telefon) VALUES (" +
						"\"" + kontakt.getVorname() + "\", " +
						"\"" + kontakt.getNachname() + "\", " +
						"\"" + kontakt.getStrasse() + "\", " +
						"\"" + kontakt.getHausnummer() + "\", " +
						"\"" + kontakt.getPlz() + "\", " +
						"\"" + kontakt.getOrt() + "\", " +
						"\"" + kontakt.getTelefon() + "\" " +
						");";
			}
			new Fehler().debug(insertKontakt, false);

			stmt.execute(insertKontakt);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public void deleteKontakt(String telefon) {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		try {
			Statement stmt = this.con.createStatement();

			String sql = "  DELETE FROM view_kontakte" +
					"       WHERE telefon=\"" + telefon + "\";";
			stmt.execute(sql);

		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public Kontakt getKontakt(@NotNull String telefon) {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 3);

		Kontakt kontakt = new Kontakt();
		String sql;

		try {
			Statement stmt = this.con.createStatement();

			if (telefon.equals("first")) {
				sql = "SELECT * " +
						"FROM view_kontakte " +
						"LIMIT 1;";
			} else {
				sql = "SELECT * " +
						"FROM view_kontakte " +
						"WHERE telefon=\"" + telefon + "\";";
			}
			ResultSet rs = stmt.executeQuery(sql);

			while (rs.next()) {
				if (rs.getString("telefon").equals(telefon) || telefon.equals("first")) {
					kontakt.setId(rs.getString("telefon"));
					kontakt.setVorname(rs.getString("vorname"));
					kontakt.setNachname(rs.getString("nachname"));
					kontakt.setStrasse(rs.getString("strasse"));
					kontakt.setHausnummer(rs.getString("hnr"));
					kontakt.setPlz(rs.getString("plz"));
					kontakt.setOrt(rs.getString("ort"));
					kontakt.setTelefon(rs.getString("telefon"));

					if (telefon.equals("first")) {
						break;
					}
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return kontakt;
	}

	public String[] getNavi(String telefon) {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 3);

		String[] naviResult = new String[3];

		try {
			Statement stmt = this.con.createStatement();
			String sql = "SELECT " +
					"    CASE " +
					"        WHEN LAG(telefon) OVER (ORDER BY nachname) IS NULL THEN ( " +
					"            SELECT telefon " +
					"            FROM view_kontakte " +
					"            ORDER BY nachname DESC " +
					"            LIMIT 1 " +
					"        ) " +
					"        ELSE LAG(telefon) OVER (ORDER BY nachname) " +
					"    END last, " +
					"    telefon AS current, " +
					"    CASE " +
					"        WHEN LEAD(telefon) OVER (ORDER BY nachname) IS NULL THEN ( " +
					"            SELECT telefon " +
					"            FROM view_kontakte " +
					"            ORDER BY nachname ASC " +
					"            LIMIT 1 " +
					"        ) " +
					"        ELSE LEAD(telefon) OVER (ORDER BY nachname) " +
					"    END next " +
					"FROM view_kontakte " +
					"ORDER BY nachname ASC;";

			ResultSet rs = stmt.executeQuery(sql);

			while (rs.next()) {
				if (rs.getString("current").equals(telefon)) {
					naviResult[0] = rs.getString("last");
					naviResult[1] = rs.getString("current");
					naviResult[2] = rs.getString("next");
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return naviResult;
	}

	public int countKontakte() {
		new Fehler().debug(this.getClass().getName() + "." + new Throwable().getStackTrace()[0].getMethodName(), true, 2);

		int intResult = 0;

		try {
			Statement stmt = this.con.createStatement();

			String sql = "  SELECT COUNT(*) AS count" +
					"       FROM view_kontakte;";
			ResultSet rs = stmt.executeQuery(sql);

			intResult = rs.getInt("count");

		} catch (SQLException e) {
			e.printStackTrace();
		}

		new Fehler().debug("Result: " + intResult, false);

		return intResult;
	}
}
